import typing
from asyncio import create_subprocess_shell, gather, run, subprocess
from typing import Callable, List, Tuple

import yaml


class type_os_res(typing.NamedTuple):
    stdout: str
    stderr: str
    cod: str
    cmd: str


def getInventory(path_to_inventory: str = None):
    if not path_to_inventory:
        path_to_inventory = "inventory.yml"
    with open(path_to_inventory, "r") as file:
        inventory = yaml.safe_load(file)
    return inventory


def divide_array(array: List, num_elements: int) -> List[Tuple[str, str]]:
    """Поделить переданный массив на указанное количество частей"""
    result = []
    for i in range(0, len(array), num_elements):
        result.append(array[i : i + num_elements])
    return result


def os_exe_async(
    commands: List[Tuple[str, str]],
    handle: Callable = type_os_res,
    divide_list: int = 10,
) -> List[type_os_res]:
    """
    Выполнить асинхронно команды OS, каждая команда в отдельной потоке
    """

    async def __self(
        _command: str,
        _label: str,
        _handle: Callable,
    ):
        # Выполняем команду
        proc = await create_subprocess_shell(
            cmd=_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        stdout, stderr = await proc.communicate()
        print(":>", _label)
        return _handle(
            label=_label,
            stdout=stdout.decode(),
            stderr=stderr.decode(),
            cod=proc.returncode,
            cmd=_command,
        )

    async def __loop(_cmds: List[Tuple[str, str]]):
        _task = (__self(_cmd, _label, handle) for _cmd, _label in _cmds)
        return await gather(*_task)

    da = divide_array(commands, divide_list)
    da_len = len(da)
    for index, cmd in enumerate(da):
        print(f"{index+1}/{da_len}")
        run(__loop(cmd))



import os
import pathlib

from invoke import Collection, Result, task

from .selflib import getInventory, os_exe_async


INVENTORY = getInventory(os.getenv("INVENTORY_FILE"))


@task
def export_all_jobs(ctx, server: str):
    """Экспортировать все Job с Jenkins"""
    base_command_cli = INVENTORY[server]["BASE_COMMAND_CLI"].format(**INVENTORY[server])
    pathlib.Path("data", server, "jobs").mkdir(parents=True, exist_ok=True)
    # Получить список всех Job
    jobs: Result = ctx.run(
        f"{base_command_cli} list-jobs",
    ).stdout.split("\n")
    # Перебрать в цикле все job и скачать их в XML
    commands = [(f'{base_command_cli} get-job "{job}"', job) for job in jobs if job]
    print("count jobs: ", len(commands))
    # Выполнить команды асинхронно
    os_exe_async(
        commands,
        handle=lambda label, stdout, stderr, cod, cmd: pathlib.Path(
            "data", server, "jobs", f"{label}.xml"
        ).write_text(stdout),
    )


@task
def create_job(ctx, server: str, jobname: str, in_folder):
    """Создать указанную Job на Jenkins"""
    base_command_cli = INVENTORY[server]["BASE_COMMAND_CLI"].format(**INVENTORY[server])

    res: Result = ctx.run(
        f"{base_command_cli} create-job {jobname} < {pathlib.Path('data', in_folder,'jobs',f'{jobname}.xml')}",
    )
    print(res)


@task
def update_job(ctx, server: str, jobname: str, in_folder):
    """Обновить указанный Job в Jenkins"""
    base_command_cli = INVENTORY[server]["BASE_COMMAND_CLI"].format(**INVENTORY[server])

    res: Result = ctx.run(
        f"{base_command_cli} update-job {jobname} < {pathlib.Path('data',in_folder,'jobs',f'{jobname}.xml')}",
    )
    print(res)


@task
def export_all_plugins(ctx, server: str):
    """Получить список всех плагинов"""
    base_command_cli = INVENTORY[server]["BASE_COMMAND_CLI"].format(**INVENTORY[server])

    res: Result = ctx.run(
        f"{base_command_cli} list-plugins",
    )
    print(res)
    if not res.stderr:
        pathlib.Path("data", server, "plains.txt").write_text(res.stdout)


@task
def install_plugins(ctx, server: str, plugin: str):
    """Установить указанный плагин"""
    base_command_cli = INVENTORY[server]["BASE_COMMAND_CLI"].format(**INVENTORY[server])

    res: Result = ctx.run(
        f"{base_command_cli} install-plugin {plugin}",
    )
    print(res)


@task
def my_task(ctx):
    print("!!")
    ctx.run("ls")


job_nsp = Collection()
job_nsp.add_task(export_all_jobs, "export-all")
job_nsp.add_task(create_job, "create")
job_nsp.add_task(update_job, "update")

plugins_nsp = Collection()
plugins_nsp.add_task(export_all_plugins, "export-all")
plugins_nsp.add_task(install_plugins, "install")
namespace = Collection()
namespace.add_collection(job_nsp, "job")
namespace.add_collection(plugins_nsp, "plugins")
namespace.add_task(my_task)
